/*******************************************************************************************************
 *   				                     TRABAJO FIN DE GRADO                                          *
 *******************************************************************************************************
 *   				                                                                                   *
 *   				      Desarrollo de un Sistema de Comunicacion Digital via Radio                   *
 *   				                     para Entornos Industriales                                    *
 *                                                                                                     *
 *******************************************************************************************************
 * @author:       Javier Santos Romo                                                                   *
 *******************************************************************************************************
 * @brief:        Main program and subfunctions                                                        *
 ******************************************************************************************************
 * Revision history:        17/06/2015                                                                 *
 ******************************************************************************************************/
 
 /**********************************************************
                  		Header files
 **********************************************************/
 
#include "lpc11E13.h"
#include "cc2500.h"

/**********************************************************
                 Clocks and Interrupts
**********************************************************/

#define enable_interrupts() asm("cpsie i")
#define disable_interrupts() asm("cpsid i")
#define sleep_mode() asm("wfi")

#define PCLK 48000000
#define _delay_us(n) delay_loop((n*(PCLK/1000)-6000)/4000)
#define _delay_ms(n) delay_loop((n*(PCLK/1000)-6)/4)

void delay_loop(unsigned int) __attribute__ ((naked));
void delay_loop(unsigned int d)
{
	asm volatile ("Ldelay_loop: sub r0,#1");
	asm volatile ("bne Ldelay_loop");
	asm volatile ("bx lr");
}

//-------------------------------------------------------------------------------------------------------
//  int U0putch(int data)
//
//  DESCRIPTION:
//      This function is used to transmit a data through UART
//
//  ARGUMENTS:
//      Data to transfer
//-------------------------------------------------------------------------------------------------------

int U0putch(int data)
{	
	while(!(ULSR & 0x00000040));									//Wait until TEMT is empty
	UTHR = data;													//Transmit data
}

//-------------------------------------------------------------------------------------------------------
//  unsigned char U0getch()
//
//  DESCRIPTION:
//      This function is used to get a data from UART
//
//  ARGUMENTS:
//      None
//-------------------------------------------------------------------------------------------------------

unsigned char U0getch()
{
	while(!(ULSR & 0x00000001)); 									//Wait until RBR is empty
	return URBR;													//Return data
}

#define putchar(c) U0putch(c)
#include "printf.c"

/**********************************************************
                       Definitions
**********************************************************/

#define MESSAGELENGTH 16

unsigned char buf_TX[64]={0};
unsigned char buf_RX[64]={0};

const unsigned char address_slave[4] __attribute__((section(".vectors"))) ="SgCr";

volatile unsigned char chip_select = 0;			//Chip is selected if user has pushed PIO0_1 low

volatile unsigned char wake_up = 0;				//Inform when radio is woke up

typedef enum Commands {READPINS = '1', BROADCAST, SETPIN, RESETPIN, TEMPERATURE_SENSOR, RECOVERCONFIG} Commands;

Commands commands;

const unsigned char RFSettings[] __attribute__((section(".vectors"))) ={
					0x01,  // IOCFG2              GDO2Output Pin Configuration 
					0x2E,  // IOCFG1              GDO1Output Pin Configuration 
					0x80,  // IOCFG0              GDO0Output Pin Configuration 
					0x07,  // FIFOTHR             RX FIFO and TX FIFO Thresholds
					0xD3,  // SYNC1               Sync Word, High Byte 
					0x91,  // SYNC0               Sync Word, Low Byte 
					0x10,  // PKTLEN              Packet Length (16 bits)
					0x0E,  // PKTCTRL1            Packet Automation Control
					0x44,  // PKTCTRL0            Packet Automation Control
					0x53,  // ADDR                Device Address 
					0x00,  // CHANNR              Channel Number 
					0x0A,  // FSCTRL1             Frequency Synthesizer Control 
					0x00,  // FSCTRL0             Frequency Synthesizer Control 
					0x5D,  // FREQ2               Frequency Control Word, High Byte 
					0x93,  // FREQ1               Frequency Control Word, Middle Byte 
					0xB1,  // FREQ0               Frequency Control Word, Low Byte 
					0x2D,  // MDMCFG4             Modem Configuration 
					0x3B,  // MDMCFG3             Modem Configuration 
					0x73,  // MDMCFG2             Modem Configuration
					0x22,  // MDMCFG1             Modem Configuration
					0xF8,  // MDMCFG0             Modem Configuration 
					0x00,  // DEVIATN             Modem Deviation Setting 
					0x07,  // MCSM2               Main Radio Control State Machine Configuration 
					0x0F,  // MCSM1               Main Radio Control State Machine Configuration
					0x18,  // MCSM0               Main Radio Control State Machine Configuration 
					0x1D,  // FOCCFG              Frequency Offset Compensation Configuration
					0x1C,  // BSCFG               Bit Synchronization Configuration
					0xC7,  // AGCCTRL2            AGC Control
					0x00,  // AGCCTRL1            AGC Control
					0xB0,  // AGCCTRL0            AGC Control
					0x87,  // WOREVT1             High Byte Event0 Timeout 
					0x6B,  // WOREVT0             Low Byte Event0 Timeout 
					0xF8,  // WORCTRL             Wake On Radio Control
					0xB6,  // FREND1              Front End RX Configuration 
					0x10,  // FREND0              Front End TX configuration 
					0xEA,  // FSCAL3              Frequency Synthesizer Calibration 
					0x0A,  // FSCAL2              Frequency Synthesizer Calibration 
					0x00,  // FSCAL1              Frequency Synthesizer Calibration 
					0x11,  // FSCAL0              Frequency Synthesizer Calibration 
					0x41,  // RCCTRL1             RC Oscillator Configuration 
					0x00,  // RCCTRL0             RC Oscillator Configuration 
					0x59,  // FSTEST              Frequency Synthesizer Calibration Control 
					0x7F,  // PTEST               Production Test 
					0x3F,  // AGCTEST             AGC Test 
					0x88,  // TEST2               Various Test Settings 
					0x31,  // TEST1               Various Test Settings 
					0x0B,  // TEST0               Various Test Settings 
							};

/**********************************************************
                       SPI Functions
**********************************************************/

//-------------------------------------------------------------------------------------------------------
//  void SPIconfig()
//
//  DESCRIPTION:
//      This function is used to configure the LPC11E13 SPI0
//
//  ARGUMENTS:
//      None
//-------------------------------------------------------------------------------------------------------

void spiconfig()
{
	/* SSP0 Configuration */
	PRESETCTRL = 1;													//SSP0 reset de-asserted
	SSP0CLKDIV = 1;													//SSP0 PCLOCK = 48 MHz
	
	PIO0_22 = 0x00000000;											//GD0 is GPIO (Input by defect)
	PIO1_21 = 0x00000000;											//GD2 is GPIO (Input by defect)
	
	SSP0_CR0 = 0x00000107;											//8 bit transfer, CPHA=0, CPOL=0, SPI frame format and SCR = 1
	SSP0_CPSR = 4;													//Clock Rate = 48 MHz /(4*(SCR+1)) --> 6 MHz (supported by CC2500)
	SSP0_CR1 = (1<<1);												//Mode Master and SSP0 Module enabled
	PIO0_8 = 0x00000001;											//PIO0_8 is MISO0
	PIO0_9 = 0x00000001;											//PIO0_9 is MOSI0
	SWCLK_PIO0_10 = 0x00000002;										//PIO0_10 is SCK0
	PIO1_31 = 0x00000000;											//PIO1_31 is GPIO (CSn)
	DIR1 |= (1<<31);			   								    //PIO1_31 is output
	SET1 = (1<<31);													//PIO1_31 is "1" (SS inactive)
}

//-------------------------------------------------------------------------------------------------------
//  unsigned int spixfer(unsigned int d)
//
//  DESCRIPTION:
//      This function is used to transfer a data to CC2500
//      (Its suppossed that /SS is active before calling this function)
//  ARGUMENTS:
//      Data to tranfer
//-------------------------------------------------------------------------------------------------------

unsigned int spixfer(unsigned char data)
{
	while(!(SSP0_SR&(0x00000002)));									// Wait until TX FIFO is empty
	SSP0_DR=data; 													// Begin the transfer
	while(SSP0_SR&(0x00000010));									// Wait until finish the transfer (BSY==0)
	return SSP0_DR;													// Return the data received (TFE==0)
}	

//-------------------------------------------------------------------------------------------------------
//  unsigned char strobe(unsigned char addr)
//
//  DESCRIPTION:
//      This function is used to transfer a strobe to CC2500
//  ARGUMENTS:
//      Strobe
//-------------------------------------------------------------------------------------------------------

unsigned char strobe(unsigned char addr)
{
	unsigned char st;
	CLR1 = (1<<31);													//Put SS low
	while(W0_8);													//MCU must wait until the SO pin from CC2500 goes low before
																	//starting to transfer the strobe byte. This indicates that 
																	//the voltage regulator has stabilized and the crystal is running.
	st=spixfer(addr);												//Transfer the strobe
	SET1 = (1<<31);													//Release SS
	return st;														//Return Status Byte
}

//-------------------------------------------------------------------------------------------------------
//  void regwr(unsigned char addr, unsigned char data)
//
//  DESCRIPTION:
//      This function is used to write a register of CC2500
//  ARGUMENTS:
//      Register address and data
//-------------------------------------------------------------------------------------------------------

void regwr(unsigned char addr, unsigned char data)
{
	CLR1 = (1<<31);													//Put SS low
	while(W0_8);													//MCU must wait until the SO pin from CC2500 goes low before
																	//starting to transfer the strobe byte. This indicates that 
																	//the voltage regulator has stabilized and the crystal is running.
	spixfer(addr);													//Transfer the address
	spixfer(data);													//Transfer the data
	SET1 = (1<<31);													//Release SS
}

//-------------------------------------------------------------------------------------------------------
//  unsigned char regrd(unsigned char addr)
//
//  DESCRIPTION:
//      This function is used to read a register of CC2500 
//  ARGUMENTS:
//      Register address
//-------------------------------------------------------------------------------------------------------

unsigned char regrd(unsigned char addr)
{
	unsigned char data;
	CLR1 = (1<<31);													//Put SS low
	while(W0_8);													//MCU must wait until the SO pin from CC2500 goes low before
																	//starting to transfer the strobe byte. This indicates that 
																	//the voltage regulator has stabilized and the crystal is running.																		
	spixfer(addr|READ_SINGLE);										//Transfer the address
	data=spixfer(0);												//CC2500 transfer the data
	SET1 = (1<<31);													//Release SS
	return data;													//Return Register
}

//-------------------------------------------------------------------------------------------------------
//  void burstwr(unsigned char addr,unsigned char *src,unsigned char length)
//
//  DESCRIPTION:
//      This function is used to write a set of parameters in burst mode
//  ARGUMENTS:
//      Start address, Set of data and number of data
//-------------------------------------------------------------------------------------------------------

void burstwr(unsigned char addr,unsigned char *src,unsigned char length)
{
	CLR1 = (1<<31);													//Put SS low
	while(W0_8);													//MCU must wait until the SO pin from CC2500 goes low before
																	//starting to transfer the strobe byte. This indicates that 
																	//the voltage regulator has stabilized and the crystal is running.
	spixfer(addr|WRITE_BURST);										//Transfer the first address
	while(length--) spixfer(*src++);								//Transfer all the data since the first address
	SET1 = (1<<31);													//Release SS
}

//-------------------------------------------------------------------------------------------------------
//  void burstrd(unsigned char addr,unsigned char *dst,unsigned char length)
//
//  DESCRIPTION:
//      This function is used to read a set of parameters in burst mode
//  ARGUMENTS:
//      Start address, Set of address and number of data
//-------------------------------------------------------------------------------------------------------

void burstrd(unsigned char addr,unsigned char *dst,unsigned char length)
{
	CLR1 = (1<<31);													//Put SS low
	while(W0_8);													//MCU must wait until the SO pin from CC2500 goes low before
																	//starting to transfer the strobe byte. This indicates that 
																	//the voltage regulator has stabilized and the crystal is running.
	spixfer(addr|READ_BURST);										//Transfer the first address
	while(length--) *dst++=spixfer(0);								//Transfer all the data since the first address
	SET1 = (1<<31);													//Release SS
}

//-------------------------------------------------------------------------------------------------------
//  void lockpll()
//
//  DESCRIPTION:
//      This function is used to lock the PLL
//  ARGUMENTS:
//      None
//-------------------------------------------------------------------------------------------------------

void lockpll()
{
	while(regrd(FSCAL1)==0x3F);		//PLL will be locked if FSCAL1!=0x3F
}

//-------------------------------------------------------------------------------------------------------
//  void ccreset()
//
//  DESCRIPTION:
//      This function is used to iniciate CC2500 after a Power On Reset
//  ARGUMENTS:
//      None
//-------------------------------------------------------------------------------------------------------

void ccreset()
{
	CLR1 = (1<<31);													//Put SS low
	_delay_us(1);
	SET1 = (1<<31);													//Release SS
	_delay_us(40);
	CLR1 = (1<<31);													//Put SS low
	while(W0_8);
	spixfer(SRES);
	SET1 = (1<<31);													//Release SS
}

//-------------------------------------------------------------------------------------------------------
//  void makeframe(unsigned char *buff, unsigned char *command, unsigned char len)
//
//  DESCRIPTION:
//      This function is used to copy a set of parameters from one address to another.
//  ARGUMENTS:
//      Destiny address, Source address and number of parameters
//-------------------------------------------------------------------------------------------------------

void makeframe(unsigned char *buff, unsigned char *command, unsigned char len)
{
	while(len--)
		*buff++ = *command++;
}

//-------------------------------------------------------------------------------------------------------
//  void initialize_vector(unsigned char *buff, unsigned char length)
//
//  DESCRIPTION:
//      This function is used to initialize a vector
//  ARGUMENTS:
//      Vector to initialize and its length 
//-------------------------------------------------------------------------------------------------------

void initialize_vector(unsigned char *buff, unsigned char length)
{
	while(length--)
		*buff++ = 0;	
}

//-------------------------------------------------------------------------------------------------------
//  unsigned char check_address(unsigned char *slave, unsigned char *received, unsigned char length)
//
//  DESCRIPTION:
//      This function is used to check if an received address is correct
//  ARGUMENTS:
//      Address of slave, address received and length of the address
//-------------------------------------------------------------------------------------------------------

unsigned char check_address(unsigned char *slave, unsigned char *received, unsigned char length)
{
	unsigned char length2 = length, OK_1 = 1, OK_2 = 1;
	unsigned char *received2 = received;
	
	while(length--)													//While loop for checking hardcoded address 
	{
		if(*slave++ != *received++)									//If address isnt equal
		{
			OK_1 = 0;
			break;	
		}
	}
	
	while(length2--)												//While loop for checking broadcast address
	{
		if(*received2++)											//If address isnt equal
		{
			OK_2 = 0;
			break;
		}
	}
	
	if(OK_1 || OK_2)					
		return 1;													//If address is correct returns 1
	else
		return 0;
}

//-------------------------------------------------------------------------------------------------------
//  unsigned int read_sensor()
//
//  DESCRIPTION:
//      This function is used to read the measure of the analog temperature sensor
//  ARGUMENTS:
//     None
//-------------------------------------------------------------------------------------------------------

unsigned int read_sensor()
{
	unsigned int sensor_value;
	ADCCR |= (1<<24);												//Start the conversion
	while(!(ADCDR6&(1<<31)));										//When bit 31 (DONE) is set, conversion has finished
	sensor_value = ((ADCDR6>>6)&0x000003FF);						//Store the value captured by the sensor (10 bit resolution)
	ADCCR &=~(1<<24);												//Stop the conversion
	
	// V_sensor = 2.43mV/C * Temperature_sensor + 750mV
	// V_sensor = 3.3V / 1024 * sensor_value
	return sensor_value;
}

/**********************************************************
                       INTERRUPT SECTION
**********************************************************/

void PIN_INT0_interrupt(void)
{
	//We'll enter here when PIO0_1 is pushed LOW
	FALL = 1;														//PINTSEL0 falling-edge is cleared
	
	_printf("\n\nDentro de la rutina PINTSEL\n\n");
	
	chip_select = 1;
	CT16B0_TCR = 0x01; 												//Timer begins to count	
}

void CT16B0_interrupt(void)
{	
	CT16B0_IR = (1<<1);												//Clear flag for match channel 1
	CT16B0_TCR = 0x00;												//Stop Timer
	CT16B0_TC = 0;													//Reset Timer
	
	_printf("\nEn la rutina Timer\n");
	
	chip_select = 0;
}

void CT16B1_interrupt(void)
{	
	CT16B1_IR = (1<<1);												//Clear flag for match channel 1
	CT16B1_TCR = 0x00;
	CT16B1_TC = 0;													//Reset Timer
	
	if(wake_up) 													//If radio is woke up
	{
		CT16B1_MR1 = 1440;											//300ms asleep
		strobe(SIDLE);												//Radio goes sleep from IDLE 
		strobe(SFRX);                   								//Flush the FIFOs
		strobe(SFTX);
		strobe(SPWD);												//Radio goes sleep	
	}
	else 
	{
		CT16B1_MR1 = 480;											//100ms woke up
		strobe(SRX);												//Radio wakes up and goes RX mode		
	}
	wake_up = !wake_up;												//Invert the status
	
	CT16B1_TCR = 0x01;												//Timer begins
}

/**********************************************************
                       MAIN SECTION
**********************************************************/

int main (void) 
{
	unsigned char i,num,pin; 
	unsigned char port = 0x00;										//8 bits for the pins
	unsigned char* ptr_port = &port;
	unsigned char temperature[3]={0};

	//address_slave[1] = 0x00;
	
	disable_interrupts();
	
	SYSAHBCLKCTRL |= (1<<6)|(1<<7)|(1<<8)|(1<<11)|(1<<12)|(1<<13)|(1<<16)|(1<<19);	//Turn on clock for GPIO, CT16B0, CT16B1, SSP0, USART, ADC, IOCON and GPIOPINT  
	SYSAHBCLKCTRL &= ~(1<<5);										//Turn off clock for I2C
	
	PIO0_18 = 0x00000000;											//UART_ISP cabled is GPIO now (Error in the prototype)
	PIO0_19 = 0x00000000;
	
	/************* PIO0_1 is a Pin Interrupt ***********************/
	//PIO0_1 (ISP) is GPIO (Input by defect with pull-up enabled)
	
	PINTSEL0 = 1;													//PINTSEL0 is interrupt for PIO0_1
	
	ISEL = 0;														//PINTSEL0 is edge sensitive
	IENF = 1;														//PINTSEL0 is falling-edge sensitive enabled
	SIENF = 1;														//PINTSEL0 is updated like in IENF
	FALL = 1;														//PINTSEL0 falling-edge is cleared
	
	ISER0 = 1;														//PINTSEL0 enabled in NVIC
	//IPR0 |= 0x00000000;											//PINTSEL0 is highest priority
	
	/************* USART Configuration **************************/	
	PIO1_26 = 0x00000002;											//PIO1_26 is RXD 
	PIO1_27 = 0x00000002;											//PIO1_27 is TXD
	
	UARTCLKDIV = 1;													//UART PCLOCK = 48 MHz
	
	ULCR = 0x83;    												// 10000011 Line Control Register
																	//       .. Character length 11 = 8 bits
																	//      .   Stop bits         0 = 1 bit
																	//     .    Parity enable     0 = No parity
																	//    .     Even parity       non sense
																	//   .      Stick parity      non sense
																	//  .       Break control     0 = No break
																	// .        DLAB              1 = Access DLM,DLL	
		
	/* Baud Rate = 48MHz/(16*17*(1+8/15)) = 115089.5 Baudios */
	UDLL = 17; 														//8 bits LSB for Baud Rate Generator												
	UDLM = 0;  														//8 bits MSB for Baud Rate Generator
	UFDR = 0x000000F8;    											//DivAddVal = 8 and MulVal = 15
	
	ULCR = 0x03; 													//  [7]   DLAB = 0

	UFCR = 0x07;    												// 00000111 FIFO Control Register
																	//        . FIFO enable       1 = TX&RX FIFO on 
																	//       .  RX FIFO Reset     1 = Reset RX FIFO
																	//      .   TX FIFO Reset     1 = Reset TX FIFO
																	// ..       RX int.t trigger level 00 = 1 byte		

	/************* GPIO Inputs and Outputs Configurator ***********************/
	PIO1_14 = 0x00000000;											//PIO1_14 is GPIO
	PIO1_22 = 0x00000000;											//PIO1_22 is GPIO
	SWDIO_PIO0_15 = 0x00000081;										//PIO0_15 is GPIO and Digital Mode
	PIO0_16 = 0x00000080;											//PIO0_16 is GPIO and Digital Mode
	DIR0 |=(1<<15)|(1<<16);											//Pins as outputs
	DIR1 |=(1<<14)|(1<<22);
	
	PIO0_17 = 0x00000000;											//PIO0_17 is GPIO
	PIO0_18 = 0x00000000;											//PIO0_18 is GPIO
	PIO0_19 = 0x00000000;											//PIO0_19 is GPIO
	PIO1_16 = 0x00000000;											//PIO1_16 is GPIO
	DIR0 &=~(1<<17);												//Pins as inputs
	DIR0 &=~(1<<18);
	DIR0 &=~(1<<19);
	DIR1 &=~(1<<16);
	
	DIR0|=(1<<3);//salida PIO0_3 solamente para test
	
	/***************** Timer 0 Configurator ************************/
	CT16B0_PR = 14999; 												//Pre-escale 15000
	
	CT16B0_TC = 0; 													//Sure TC is 0
	
	CT16B0_MR1 = 48000; 											//Timer will count 15 s
	
	CT16B0_MCR = (1<<3);											//Interrupt will happen when TC reach MR1

	ISER0 = (1<<16);												//CT16B0 interrupt enabled in NVIC
	
	/***************** Timer 1 Configurator ************************/
	CT16B1_PR = 9999; 												//Pre-escale 10000
	
	CT16B1_TC = 0; 													//Sure TC is 0
	
	CT16B1_MR1 = 1440; 												//Timer will count 300ms
	
	CT16B1_MCR = (1<<3);											//Interrupt will happen when TC reach MR1

	ISER0 = (1<<17);												//CT16B0 interrupt enabled in NVIC
	
	/***************** ADC Configurator ************************/
	PIO0_22 = 0x00000001;											//GD0 is input of analog temperature sensor (AD6)
	ADCCR = 0x00000B40;												//ADC->4MHz, BURST->0, START->0, AD6
	
	/////////////// Main program ////////////////////////
			
	TDI_PIO0_11 = 0x00000001;
	DIR0 |= (1<<11);
	W0_11 = 1;
		
	_printf("\n\nPrograma main\n");
	
	spiconfig();

	ccreset();														//When Reset is completed CC2500 is in IDLE State

	_printf("\nMARCSTATE=%02X\n",regrd(MARCSTATE));
	
	burstwr(0x00,RFSettings,47);									//Register Configuration for CC2500 from SmartRF Studio
	
	regwr(PATABLE,0xFF);											//Output Power +1 dBm @21.5 mA	
	
	_printf("\n\nConfiguracion completa\n\n");
	
	strobe(SFRX);                   								//Flush the FIFOs
    strobe(SFTX);

	strobe(SPWD);													//Radio goes sleep mode
	
	//If an interrupt is not enabled, asserting its interrupt signal changes the interrupt state to
	//pending, but the NVIC never activates the interrupt, regardless of its priority.
	FALL = 1;														//PINTSEL0 falling-edge is cleared
	ICPR0 = 1;														//Clear pending interrupt in PINTSEL0
	RISE |= (1<<1);													//PINTSEL1 rising-edge is cleared
	ICPR0 = (1<<1);													//Clear pending interrupt in PINTSEL1

	enable_interrupts();
	
	/***************** Sleep Mode Configurator ************************/
	PCON = 0x00000000;												//Sleep mode indicated to enter
	SCR &=~(1<<2);													//SLEEPDEEP bit introduced
	
	CT16B1_TCR = 0x01;												//Timer begins
	sleep_mode();													//Sleep mode entered of MCU
	
	while(1)
	{
		while(wake_up)
		{
			if(W1_21)													//If received packet
			{
				disable_interrupts();
				
				_printf("\n\nMARCSTATE=%02X\n",regrd(MARCSTATE));	
								
				num=regrd(RXBYTES)&0x7F;								//Read NUM_RXBYTES
				
				_printf("BYTES EN RX FIFO=%d\n",num);
				
				if(num>0)												//Indicate that CRC was OK and data can be read from RX FIFO
				{
					burstrd(RXFIFO,buf_RX,num);	
					
					if(check_address(address_slave,buf_RX,4))			//If address of slave or broadcast address received
					{		
						_printf("MENSAJE RECIBIDO\n");
						for (i=0;i<num;i++) U0putch(buf_RX[i]);
							
						commands = buf_RX[8];
						
						_printf("\nMENSAJE ENVIADO\n");
						
						switch((char)commands)								//Type of command
						{
							case READPINS:									// I/O Command	
								port =  (B1_16 & 0x01);						//Bit 0 is PIO1_16
								port|= ((B0_19 & 0x01)<<1);					//Bit 1 is PIO0_19
								port|= ((B0_18 & 0x01)<<2);					//Bit 2 is PIO0_18
								port|= ((B0_17 & 0x01)<<3);					//Bit 3 is PIO0_17
								port|= ((B0_16 & 0x01)<<4);					//Bit 4 is PIO0_16
								port|= ((B0_15 & 0x01)<<5);					//Bit 5 is PIO0_15
								port|= ((B1_22 & 0x01)<<6);					//Bit 6 is PIO1_22
								port|= ((B1_14 & 0x01)<<7);					//Bit 7 is PIO1_14
								
								makeframe(buf_TX,&buf_RX[4],4);				//Copy in &buf_TX[0] the address of the master
								makeframe(&buf_TX[4],address_slave,4);		//Copy in &buf_TX[4] the address of the slave
								makeframe(&buf_TX[8],&buf_RX[8],1);			//Copy in &buf_TX[8] the type of command
								makeframe(&buf_TX[14],ptr_port,1);			//Copy in &buf_TX[14] port								
								makeframe(&buf_TX[15],&buf_RX[15],1);		//End of Frame
								burstwr(TXFIFO,buf_TX,regrd(PKTLEN));
							
								strobe(STX);       							//Switch to RX once the message is sent (MCSM1)
								for (i=0;i<num;i++) U0putch(buf_TX[i]);	
							break;
								
							case BROADCAST:									// Address Command	
								if(chip_select)								//Only if PIO0_1 was pushed LOW the Slave answers
								{	
									makeframe(buf_TX,&buf_RX[4],4);			//Copy in &buf_TX[0] the address of the master
									makeframe(&buf_TX[4],address_slave,4);	//Copy in &buf_TX[4] the address of the slave
									makeframe(&buf_TX[8],&buf_RX[8],1);		//Copy in &buf_TX[8] the type of command
									makeframe(&buf_TX[9],address_slave,4);	//Copy in &buf_TX[9] the address of the slave
									makeframe(&buf_TX[15],&buf_RX[15],1);	//End of Frame
									burstwr(TXFIFO,buf_TX,regrd(PKTLEN));
									
									strobe(STX);       						//Switch to RX once the message is sent (MCSM1)
									for (i=0;i<num;i++) U0putch(buf_TX[i]);
								}
							break;
	
							case SETPIN:									// Set Pin	
								pin = (buf_RX[13]-48)*10 + (buf_RX[14]-48);	//Pin number in decimal
								
								if(pin<=23)									//If we are in PORT0
								{
									SET0 =(1<<pin);							//Pin pin is set
								}
								else										//If we are in PORT1
								{
									pin = pin-24;							//Rest PORT0
									SET1 =(1<<pin);							//Pin pin is set							
								}
								
								makeframe(buf_TX,&buf_RX[4],4);				//Copy in &buf_TX[0] the address of the master
								makeframe(&buf_TX[4],address_slave,4);		//Copy in &buf_TX[4] the address of the slave
								makeframe(&buf_TX[8],&buf_RX[8],1);			//Copy in &buf_TX[8] the type of command
								makeframe(&buf_TX[9],(unsigned char*)"ACKACK",6);//Copy in &buf_TX[9] the Set Pin ACK
								makeframe(&buf_TX[15],&buf_RX[15],1);		//End of Frame
								burstwr(TXFIFO,buf_TX,regrd(PKTLEN));
							
								strobe(STX);       							//Switch to RX once the message is sent (MCSM1)
								for (i=0;i<num;i++) U0putch(buf_TX[i]);								
							break;
								
							case RESETPIN:									//Reset Pin	
								pin = (buf_RX[13]-48)*10 + (buf_RX[14]-48);	//Pin number in decimal
								
								if(pin<=23)									//If we are in PORT0
								{
									CLR0 =(1<<pin);							//Pin pin is reset
								}
								else										//If we are in PORT1
								{
									pin = pin-24;							//Rest PORT0
									CLR1 =(1<<pin);							//Pin pin is reset							
								}
							
								makeframe(buf_TX,&buf_RX[4],4);				//Copy in &buf_TX[0] the address of the master
								makeframe(&buf_TX[4],address_slave,4);		//Copy in &buf_TX[4] the address of the slave
								makeframe(&buf_TX[8],&buf_RX[8],1);			//Copy in &buf_TX[8] the type of command
								makeframe(&buf_TX[9],(unsigned char*)"ACKACK",6);//Copy in &buf_TX[9] the Set Pin ACK
								makeframe(&buf_TX[15],&buf_RX[15],1);		//End of Frame
								burstwr(TXFIFO,buf_TX,regrd(PKTLEN));
							
								strobe(STX);       							//Switch to IDLE once the message is sent (MCSM1)
								for (i=0;i<num;i++) U0putch(buf_TX[i]);
							break;
							
							case TEMPERATURE_SENSOR:						// Analog Sensor Temperature	
								_sprintf(temperature,"%03X",read_sensor());
		
								makeframe(buf_TX,&buf_RX[4],4);				//Copy in &buf_TX[0] the address of the master
								makeframe(&buf_TX[4],address_slave,4);		//Copy in &buf_TX[4] the address of the slave
								makeframe(&buf_TX[8],&buf_RX[8],1);			//Copy in &buf_TX[8] the type of command
								makeframe(&buf_TX[12],temperature,3);		//Value from sensor								
								makeframe(&buf_TX[15],&buf_RX[15],1);		//End of Frame
								burstwr(TXFIFO,buf_TX,regrd(PKTLEN));
							
								strobe(STX);       							//Switch to RX once the message is sent (MCSM1)
								for (i=0;i<num;i++) U0putch(buf_TX[i]);
							break;
							
							case RECOVERCONFIG:								// Recover Config Command	
								B0_16 = ((buf_RX[14] & 0x10)>>4);			//PIO0_16
								B0_15 = ((buf_RX[14] & 0x20)>>5);			//PIO0_15
								B1_22 = ((buf_RX[14] & 0x40)>>6);			//PIO1_22
								B1_14 = ((buf_RX[14] & 0x80)>>7);			//PIO1_14
								
								makeframe(buf_TX,&buf_RX[4],4);				//Copy in &buf_TX[0] the address of the master
								makeframe(&buf_TX[4],address_slave,4);		//Copy in &buf_TX[4] the address of the slave
								makeframe(&buf_TX[8],&buf_RX[8],1);			//Copy in &buf_TX[8] the type of command
								makeframe(&buf_TX[9],(unsigned char*)"ACKACK",6);//Copy in &buf_TX[9] the recover config ACK								
								makeframe(&buf_TX[15],&buf_RX[15],1);		//End of Frame
								burstwr(TXFIFO,buf_TX,regrd(PKTLEN));
							
								strobe(STX);       							//Switch to RX once the message is sent (MCSM1)
								for (i=0;i<num;i++) U0putch(buf_TX[i]);	
							break;
						}	
						_printf("\nMARCSTATE=%02X\n",regrd(MARCSTATE));
						initialize_vector(buf_TX,MESSAGELENGTH);
					}
				}
				enable_interrupts();
			}
		}
		sleep_mode();													//We can back to sleep state of MCU
	}		
}

