/*******************************************************************************************************
 *   				                     TRABAJO FIN DE GRADO                                          *
 *******************************************************************************************************
 *   				                                                                                   *
 *   				      Desarrollo de un Sistema de Comunicacion Digital via Radio                   *
 *   				                     para Entornos Industriales                                    *
 *                                                                                                     *
 *******************************************************************************************************
 * @author:       Javier Santos Romo                                                                   *
 *******************************************************************************************************
 * @brief:        Main program and subfunctions                                                        *
 ******************************************************************************************************
 * Revision history:        17/06/2015                                                                 *
 ******************************************************************************************************/
 
 /**********************************************************
                  		Header files
 **********************************************************/
 
#include "lpc11E13.h"
#include "cc2500.h"

/**********************************************************
                 Clocks and Interrupts
**********************************************************/

#define enable_interrupts() asm("cpsie i")
#define disable_interrupts() asm("cpsid i")

#define PCLK 48000000
#define _delay_us(n) delay_loop((n*(PCLK/1000)-6000)/4000)
#define _delay_ms(n) delay_loop((n*(PCLK/1000)-6)/4)

void delay_loop(unsigned int) __attribute__ ((naked));
void delay_loop(unsigned int d)
{
	asm volatile ("Ldelay_loop: sub r0,#1");
	asm volatile ("bne Ldelay_loop");
	asm volatile ("bx lr");
}

//-------------------------------------------------------------------------------------------------------
//  int U0putch(int data)
//
//  DESCRIPTION:
//      This function is used to transmit a data through UART
//
//  ARGUMENTS:
//      Data to transfer
//-------------------------------------------------------------------------------------------------------

int U0putch(int data)
{	
	while(!(ULSR & 0x00000040));									//Wait until TEMT is empty
	UTHR = data;													//Transmit data
}

//-------------------------------------------------------------------------------------------------------
//  unsigned char U0getch()
//
//  DESCRIPTION:
//      This function is used to get a data from UART
//
//  ARGUMENTS:
//      None
//-------------------------------------------------------------------------------------------------------

unsigned char U0getch()
{
	while(!(ULSR & 0x00000001)); 									//Wait until RBR is empty
	return URBR;													//Return data
}

#define putchar(c) U0putch(c)
#include "printf.c"

/**********************************************************
                       Definitions
**********************************************************/

#define LENGTHFRAME 16
#define COMMANDLENGTH 12
#define MAXITER 10

unsigned char buf_TX[64]={0};
unsigned char buf_RX[64]={0};

const unsigned char address_master[4] ="maes";

volatile unsigned char iter=0;										//Number of times a message has been retransmited

volatile unsigned char received_packet = 0;							//Indicate if one packet has been received

volatile unsigned char temp_expire = 0;								//Indicate if temp has been overflowed

volatile unsigned char EOF = 'L';									//Indicate End Of Frame

volatile unsigned char message_OK = 2;								//Indicate if the message received is correct 
																	//0 -> Not OK   1 -> OK  2 -> Tristate

volatile unsigned char readcommand[COMMANDLENGTH]={0}; 				//Master reads the command from the computer

volatile unsigned char num_read_bytes = 0;			

const unsigned char RFSettings[] ={
					0x01,  // IOCFG2              GDO2Output Pin Configuration 
					0x2E,  // IOCFG1              GDO1Output Pin Configuration 
					0x3F,  // IOCFG0              GDO0Output Pin Configuration 
					0x07,  // FIFOTHR             RX FIFO and TX FIFO Thresholds
					0xD3,  // SYNC1               Sync Word, High Byte 
					0x91,  // SYNC0               Sync Word, Low Byte 
					0x10,  // PKTLEN              Packet Length (16 bytes)
					0x0E,  // PKTCTRL1            Packet Automation Control
					0x44,  // PKTCTRL0            Packet Automation Control
					0x6D,  // ADDR                Device Address 
					0x00,  // CHANNR              Channel Number 
					0x0A,  // FSCTRL1             Frequency Synthesizer Control 
					0x00,  // FSCTRL0             Frequency Synthesizer Control 
					0x5D,  // FREQ2               Frequency Control Word, High Byte 
					0x93,  // FREQ1               Frequency Control Word, Middle Byte 
					0xB1,  // FREQ0               Frequency Control Word, Low Byte 
					0x2D,  // MDMCFG4             Modem Configuration 
					0x3B,  // MDMCFG3             Modem Configuration 
					0x73,  // MDMCFG2             Modem Configuration
					0x22,  // MDMCFG1             Modem Configuration
					0xF8,  // MDMCFG0             Modem Configuration 
					0x00,  // DEVIATN             Modem Deviation Setting 
					0x07,  // MCSM2               Main Radio Control State Machine Configuration 
					0x0F,  // MCSM1               Main Radio Control State Machine Configuration
					0x18,  // MCSM0               Main Radio Control State Machine Configuration 
					0x1D,  // FOCCFG              Frequency Offset Compensation Configuration
					0x1C,  // BSCFG               Bit Synchronization Configuration
					0xC7,  // AGCCTRL2            AGC Control
					0x00,  // AGCCTRL1            AGC Control
					0xB0,  // AGCCTRL0            AGC Control
					0x87,  // WOREVT1             High Byte Event0 Timeout 
					0x6B,  // WOREVT0             Low Byte Event0 Timeout 
					0xF8,  // WORCTRL             Wake On Radio Control
					0xB6,  // FREND1              Front End RX Configuration 
					0x10,  // FREND0              Front End TX configuration 
					0xEA,  // FSCAL3              Frequency Synthesizer Calibration 
					0x0A,  // FSCAL2              Frequency Synthesizer Calibration 
					0x00,  // FSCAL1              Frequency Synthesizer Calibration 
					0x11,  // FSCAL0              Frequency Synthesizer Calibration 
					0x41,  // RCCTRL1             RC Oscillator Configuration 
					0x00,  // RCCTRL0             RC Oscillator Configuration 
					0x59,  // FSTEST              Frequency Synthesizer Calibration Control 
					0x7F,  // PTEST               Production Test 
					0x3F,  // AGCTEST             AGC Test 
					0x88,  // TEST2               Various Test Settings 
					0x31,  // TEST1               Various Test Settings 
					0x0B,  // TEST0               Various Test Settings 
							};

/**********************************************************
                       SPI Functions
**********************************************************/

//-------------------------------------------------------------------------------------------------------
//  void SPIconfig()
//
//  DESCRIPTION:
//      This function is used to configure the LPC11E13 SPI0
//
//  ARGUMENTS:
//      None
//-------------------------------------------------------------------------------------------------------

void spiconfig()
{
	/* SSP0 Configuration */
	PRESETCTRL = 1;													//SSP0 reset de-asserted
	SSP0CLKDIV = 1;													//SSP0 PCLOCK = 48 MHz
	
	PIO0_22 = 0x00000000;											//GD0 is GPIO (Input by defect)
	PIO1_21 = 0x00000000;											//GD2 is GPIO (Input by defect)
	
	SSP0_CR0 = 0x00000107;											//8 bit transfer, CPHA=0, CPOL=0, SPI frame format and SCR = 1
	SSP0_CPSR = 4;													//Clock Rate = 48 MHz /(4*(SCR+1)) --> 6 MHz (supported by CC2500)
	SSP0_CR1 = (1<<1);												//Mode Master and SSP0 Module enabled
	PIO0_8 = 0x00000001;											//PIO0_8 is MISO0
	PIO0_9 = 0x00000001;											//PIO0_9 is MOSI0
	SWCLK_PIO0_10 = 0x00000002;										//PIO0_10 is SCK0
	PIO1_31 = 0x00000000;											//PIO1_31 is GPIO (CSn)
	DIR1 |= (1<<31);			   								    //PIO1_31 is output
	SET1 = (1<<31);													//PIO1_31 is "1" (SS inactive)
}

//-------------------------------------------------------------------------------------------------------
//  unsigned int spixfer(unsigned int d)
//
//  DESCRIPTION:
//      This function is used to transfer a data to CC2500
//      (It´s suppossed that /SS is active before calling this function)
//  ARGUMENTS:
//      Data to tranfer
//-------------------------------------------------------------------------------------------------------

unsigned int spixfer(unsigned char data)
{
	while(!(SSP0_SR&(0x00000002)));									// Wait until TX FIFO is empty
	SSP0_DR=data; 													// Begin the transfer
	while(SSP0_SR&(0x00000010));									// Wait until finish the transfer (BSY==0)
	return SSP0_DR;													// Return the data received (TFE==0)
}	

//-------------------------------------------------------------------------------------------------------
//  unsigned char strobe(unsigned char addr)
//
//  DESCRIPTION:
//      This function is used to transfer a strobe to CC2500
//  ARGUMENTS:
//      Strobe
//-------------------------------------------------------------------------------------------------------

unsigned char strobe(unsigned char addr)
{
	unsigned char st;
	CLR1 = (1<<31);													//Put SS low
	while(W0_8);													//MCU must wait until the SO pin from CC2500 goes low before
																	//starting to transfer the strobe byte. This indicates that 
																	//the voltage regulator has stabilized and the crystal is running.
	st=spixfer(addr);												//Transfer the strobe
	SET1 = (1<<31);													//Release SS
	return st;														//Return Status Byte
}

//-------------------------------------------------------------------------------------------------------
//  void regwr(unsigned char addr, unsigned char data)
//
//  DESCRIPTION:
//      This function is used to write a register of CC2500
//  ARGUMENTS:
//      Register address and data
//-------------------------------------------------------------------------------------------------------

void regwr(unsigned char addr, unsigned char data)
{
	CLR1 = (1<<31);													//Put SS low
	while(W0_8);													//MCU must wait until the SO pin from CC2500 goes low before
																	//starting to transfer the strobe byte. This indicates that 
																	//the voltage regulator has stabilized and the crystal is running.
	spixfer(addr);													//Transfer the address
	spixfer(data);													//Transfer the data
	SET1 = (1<<31);													//Release SS
}

//-------------------------------------------------------------------------------------------------------
//  unsigned char regrd(unsigned char addr)
//
//  DESCRIPTION:
//      This function is used to read a register of CC2500 
//  ARGUMENTS:
//      Register address
//-------------------------------------------------------------------------------------------------------

unsigned char regrd(unsigned char addr)
{
	unsigned char data;
	CLR1 = (1<<31);													//Put SS low
	while(W0_8);													//MCU must wait until the SO pin from CC2500 goes low before
																	//starting to transfer the strobe byte. This indicates that 
																	//the voltage regulator has stabilized and the crystal is running.																		
	spixfer(addr|READ_SINGLE);										//Transfer the address
	data=spixfer(0);												//CC2500 transfer the data
	SET1 = (1<<31);													//Release SS
	return data;													//Return Register
}

//-------------------------------------------------------------------------------------------------------
//  void burstwr(unsigned char addr,unsigned char *src,unsigned char length)
//
//  DESCRIPTION:
//      This function is used to write a set of parameters in burst mode
//  ARGUMENTS:
//      Start address, Set of data and number of data
//-------------------------------------------------------------------------------------------------------

void burstwr(unsigned char addr,unsigned char *src,unsigned char length)
{
	CLR1 = (1<<31);													//Put SS low
	while(W0_8);													//MCU must wait until the SO pin from CC2500 goes low before
																	//starting to transfer the strobe byte. This indicates that 
																	//the voltage regulator has stabilized and the crystal is running.
	spixfer(addr|WRITE_BURST);										//Transfer the first address
	while(length--) spixfer(*src++);								//Transfer all the data since the first address
	SET1 = (1<<31);													//Release SS
}

//-------------------------------------------------------------------------------------------------------
//  void burstrd(unsigned char addr,unsigned char *dst,unsigned char length)
//
//  DESCRIPTION:
//      This function is used to read a set of parameters in burst mode
//  ARGUMENTS:
//      Start address, Set of address and number of data
//-------------------------------------------------------------------------------------------------------

void burstrd(unsigned char addr,unsigned char *dst,unsigned char length)
{
	CLR1 = (1<<31);													//Put SS low
	while(W0_8);													//MCU must wait until the SO pin from CC2500 goes low before
																	//starting to transfer the strobe byte. This indicates that 
																	//the voltage regulator has stabilized and the crystal is running.
	spixfer(addr|READ_BURST);										//Transfer the first address
	while(length--) *dst++=spixfer(0);								//Transfer all the data since the first address
	SET1 = (1<<31);													//Release SS
}

//-------------------------------------------------------------------------------------------------------
//  void lockpll()
//
//  DESCRIPTION:
//      This function is used to lock the PLL
//  ARGUMENTS:
//      None
//-------------------------------------------------------------------------------------------------------

void lockpll()
{
	while(regrd(FSCAL1)==0x3F);		//PLL will be locked if FSCAL1!=0x3F
}

//-------------------------------------------------------------------------------------------------------
//  void ccreset()
//
//  DESCRIPTION:
//      This function is used to iniciate CC2500 after a Power On Reset
//  ARGUMENTS:
//      None
//-------------------------------------------------------------------------------------------------------

void ccreset()
{
	CLR1 = (1<<31);													//Put SS low
	_delay_us(1);
	SET1 = (1<<31);													//Release SS
	_delay_us(40);
	CLR1 = (1<<31);													//Put SS low
	while(W0_8);
	spixfer(SRES);
	SET1 = (1<<31);													//Release SS
}

//-------------------------------------------------------------------------------------------------------
//  void makeframe(unsigned char *buff, unsigned char *command, unsigned char len)
//
//  DESCRIPTION:
//      This function is used to copy a set of parameters from one address to another.
//  ARGUMENTS:
//      Destiny address, Source address and number of parameters
//-------------------------------------------------------------------------------------------------------

void makeframe(unsigned char *buff, unsigned char *command, unsigned char len)
{
	while(len--)
		*buff++ = *command++;
}

//-------------------------------------------------------------------------------------------------------
//  void answer_to_computer(unsigned char *buff)
//
//  DESCRIPTION:
//      This function is used to transmit a set of parameters throw UART
//  ARGUMENTS:
//      Source address 
//-------------------------------------------------------------------------------------------------------

void answer_to_computer(unsigned char *buff)
{
	while(*buff != 'L')				 //End of Response
		U0putch(*buff++);		
}

//-------------------------------------------------------------------------------------------------------
//  void initialize_vector(unsigned char *buff, unsigned char length)
//
//  DESCRIPTION:
//      This function is used to initialize a vector
//  ARGUMENTS:
//      Vector to initialize and its length 
//-------------------------------------------------------------------------------------------------------

void initialize_vector(unsigned char *buff, unsigned char length)
{
	while(length--)
		*buff++ = 0;	
}

/**********************************************************
                       INTERRUPT SECTION
**********************************************************/

void USART_interrupt(void)
{
	if(ULSR & 1)												//We have received a byte
	{
		//_printf("\nEn la rutina de USART\n");
		readcommand[num_read_bytes++] = URBR;				    //readcommand save the bytes received (Clear flag)
		if(num_read_bytes == COMMANDLENGTH)						//If we reach the length of the frame
		{
			message_OK =(readcommand[num_read_bytes - 1] == EOF) ? 1 : 0;
			num_read_bytes = 0;	
		}
	}
}

void CT16B0_interrupt(void)
{
	CT16B0_IR = (1<<1);											//Clear flag for match channel 1
	//_printf("\nEn la rutina Timer\n");
	temp_expire = 1;
	iter++;
}

void PIN_INT0_interrupt(void)
{
	//We'll enter here when it was reached the final of a packet (IOCFG2=0x01) in RX FIFO
	RISE = 1;													//Clear flag of rising-edge in PINTSEL0

	CT16B0_TCR = 0x00; 											//Stop Timer
	//_printf("CT16B0_TC=%d\n", CT16B0_TC);	
	CT16B0_TC = 0;												//Reset Timer Counter
	
	//_printf("\n\nDentro de la rutina PINTSEL0\n\n");
	
	received_packet = 1;			
}

/**********************************************************
                       MAIN SECTION
**********************************************************/

int main()
{
	unsigned char i, num,lqi;
	unsigned char finished_cycle = 1;								//Inform if a cycle transmit-receive has completed

	disable_interrupts();
	
	SYSAHBCLKCTRL |= (1<<6)|(1<<7)|(1<<11)|(1<<12)|(1<<16)|(1<<19);	//Turn on clock for GPIO, CT16B0, SSP0, USART, IOCON and GPIOPINT  
	SYSAHBCLKCTRL &= ~(1<<5);										//Turn off clock for I2C
	
	PIO0_18 = 0x00000000;											//UART_ISP cabled is GPIO now (Error in the prototype)
	PIO0_19 = 0x00000000;
	
	/************* GD2 is a Pin Interrupt ***********************/
	PIO1_21 = 0x00000000;											//PIO1_21 (GD2) is GPIO (Input by defect)
	
	PINTSEL0 = 45;													//PINTSEL0 is interrupt for PIO1_21
	
	ISEL = 0;														//PINTSEL0 is edge sensitive
	IENR = 1;														//PINTSEL0 is rising-edge sensitive enabled
	SIENR = 1;														//PINTSEL0 is updated like in IENR
	RISE = 1;														//Clear flag of rising-edge in PINTSEL0
	
	ISER0 = 1;														//PINTSEL0 enabled in NVIC
	IPR0 |= 0x00000000;												//PINTSEL0 is highest priority
	
	/************* USART Configuration **************************/	
	PIO1_26 = 0x00000002;											//PIO1_26 is RXD 
	PIO1_27 = 0x00000002;											//PIO1_27 is TXD
	
	UARTCLKDIV = 1;													//UART PCLOCK = 48 MHz
	
	ULCR = 0x83;    												// 10000011 Line Control Register
																	//       .. Character length 11 = 8 bits
																	//      .   Stop bits         0 = 1 bit
																	//     .    Parity enable     0 = No parity
																	//    .     Even parity       non sense
																	//   .      Stick parity      non sense
																	//  .       Break control     0 = No break
																	// .        DLAB              1 = Access DLM,DLL	
		
	/* Baud Rate = 48MHz/(16*17*(1+8/15)) = 115089.5 Baudios */
	UDLL = 17; 														//8 bits LSB for Baud Rate Generator												
	UDLM = 0;  														//8 bits MSB for Baud Rate Generator
	UFDR = 0x000000F8;    											//DivAddVal = 8 and MulVal = 15
	
	ULCR = 0x03; 													//  [7]   DLAB = 0

	UFCR = 0x07;    												// 00000111 FIFO Control Register
																	//        . FIFO enable       1 = TX&RX FIFO on 
																	//       .  RX FIFO Reset     1 = Reset RX FIFO
																	//      .   TX FIFO Reset     1 = Reset TX FIFO
																	// ..       RX int.t trigger level 00 = 1 byte	
			  		  
	UIER = 1;   													//Interrupt on RX
	
	ISER0 = (1<<21);												//USART0 interrupt enabled in NVIC	
	
	/***************** Timer Configurator ************************/
	CT16B0_PR = 999; 												//Pre-escale 1000
	
	CT16B0_TC = 0; 													//Sure TC is 0
	
	CT16B0_MR1 = 1440; 												//Timer will count 30 ms
	
	CT16B0_MCR = (1<<3);											//Interrupt will happen when TC reach MR1

	ISER0 = (1<<16);												//CT16B0 interrupt enabled in NVIC
	
	/////////////// Main program ////////////////////////
	
	TDI_PIO0_11 = 0x00000001;
	DIR0 |= (1<<11);
	W0_11 = 1;
	
	_printf("Programa main\n");
	
	spiconfig();

	ccreset();														//When Reset is completed CC2500 is in IDLE State

	_printf("\nMARCSTATE=%02X\n",regrd(MARCSTATE));
	
	burstwr(0x00,RFSettings,47);									//Register Configuration for CC2500 from SmartRF Studio
	
	regwr(PATABLE,0xFF);											//Output Power +1 dBm @21.5 mA	
	
	_printf("\n\nConfiguracion completa\n\n");
	
	strobe(SFRX);                   								//Flush the FIFO´s
    strobe(SFTX);

	strobe(SFSTXON);												// Go to FSTXON State (To quick switch to TX)

	lockpll();														//We are sure PLL is locked

	_printf("PARTNUM=%02X\n",regrd(PARTNUM));	
	_printf("VERSION=%02X\n",regrd(VERSION));
	_printf("MARCSTATE=%02X\n",regrd(MARCSTATE));
	
	_printf("PKTLEN=%02X\n",regrd(PKTLEN));
	_printf("ADDR=%02X\n",regrd(ADDR));
	_printf("MCSM1=%02X\n",regrd(MCSM1));
	_printf("IOCFG0=%02X\n",regrd(IOCFG0));
	
	//If an interrupt is not enabled, asserting its interrupt signal changes the interrupt state to
	//pending, but the NVIC never activates the interrupt, regardless of its priority.
	RISE = 1;														//Clear flag of rising-edge in PINTSEL0
	ICPR0 = 1;														//Clear pending interrupt in PINTSEL0

	enable_interrupts();

	while(1)
	{
		if(message_OK == 1)											//If message received is correct
		{
			makeframe(buf_TX,readcommand,4);
			makeframe(&buf_TX[4],address_master,4);					//Copy in &buf_TX[4] the address of the master
			makeframe(&buf_TX[8],&readcommand[4],1);				//Copy in &buf_TX[8] the type of command
			makeframe(&buf_TX[9],&readcommand[5],6);				//6 Bytes of data
			makeframe(&buf_TX[15],&readcommand[11],1);				//End of Frame											
			burstwr(TXFIFO,buf_TX,regrd(PKTLEN));
			
			//_printf("\n\nLo que se va a enviar\n\n");
			//for (i=0;i<16;i++) U0putch(buf_TX[i]);	
			
			message_OK = 2;											//Tristate
			initialize_vector(readcommand,COMMANDLENGTH);
			finished_cycle = 0;

			CT16B0_TCR = 0x01; 										//Timer begins to count
			strobe(STX);											//Switch to RX once the message is sent (MCSM1)
		}
		else if (message_OK == 0)
		{
			message_OK = 2;											//Tristate
			initialize_vector(readcommand,COMMANDLENGTH);
			num_read_bytes = 0;	
			finished_cycle = 1;
			_printf("Error en la trama a enviar");
		}
		
		if(temp_expire)
		{
			if(iter == MAXITER)										//If we rise the maximum number of retransmitions
			{
				answer_to_computer((unsigned char*)"ErrorL");
				CT16B0_TCR = 0x00; 									//Stop the Timer
				CT16B0_TC = 0; 										//TC equals to 0
				iter = 0;											//Restart the number of retransmitions
				finished_cycle = 1;
			}
			else
			{
				//_printf("\n\nTemporizador expirado\n\n");		
				burstwr(TXFIFO,buf_TX,regrd(PKTLEN));				//Transmit again the message
				CT16B0_TC = 0;
				strobe(STX);										//Switch to RX once the message is sent (MCSM1)
			}
			temp_expire = 0;
		}

		if(received_packet)
		{	                                  			
			num=regrd(RXBYTES)&0x7F;								//Read NUM_RXBYTES
				
			//_printf("BYTES EN RX FIFO=%d\n",num);
				
			if(num>0)												//Indicates CRC is OK and data can be read from RX FIFO
			{
				burstrd(RXFIFO,buf_RX,num);
				iter = 0;											//Restart the timer
				
				//_printf("MENSAJE RECIBIDO\n");
				//for (i=0;i<num;i++) U0putch(buf_RX[i]);
					
				//_printf("\nMENSAJE PASADO A PC\n");
								
				answer_to_computer(&buf_RX[4]);						//Answer to PC
				
				finished_cycle = 1;

				i=(signed char)buf_RX[num-2];						//RSSI
				//_printf("\nI=%x\n",i);
				if(i>=128)
					i=(i-256)/2-72;     
				else
					i=i/2-72;   
				
				if(i&0x80)
				{
					i=-i;
					_printf("\nRSSI = -%d dBm\n",i);
				}
				else
					_printf("\nRSSI = +%d dBm\n",i);
		 
				_printf("LQI = %02X\n",buf_RX[num-1]&0x7F);
			}
			else
			{	
				if(iter == MAXITER)									//If we rise the maximum number of retransmitions
				{
					answer_to_computer((unsigned char*)"ErrorL");
					CT16B0_TCR = 0x00; 								//Stop the Timer
					CT16B0_TC = 0; 									//TC equals to 0
					iter = 0;										//Restart the number of retransmitions
					finished_cycle = 1;
				}
				else
				{
					if(!finished_cycle)								//Only retransmit if a cycle has not finished
					{												//Prevent of a glitch detected in PINTSEL0 (received packet)
						//_printf("\n\nCRC incorrecto, Retransmito de nuevo\n\n");
						burstwr(TXFIFO,buf_TX,regrd(PKTLEN));

						CT16B0_TCR = 0x01; 								//Restart Timer
						strobe(STX); 									//Switch to RX once the message is sent (MCSM1)
						iter++;
					}
				}	
			}
			received_packet = 0;
		}
	}
}
